// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <string>
#include <unordered_map>
#include <cassert>
#include "OrderPlacingMetaInfo.h"
#include "OrderPlacingMetaInfoDlg.h"


STDMETHODIMP COrderPlacingMetaInfoDlg::OnGWConnectionStatusChanged(eConnectionStatus newStatus)
{
   if (newStatus == eConnectionStatus::csConnectionUp)
   {
      m_spCQGCEL->put_AccountSubscriptionLevel(eAccountSubscriptionLevel::aslAccountUpdatesAndOrders);

      m_btnPlace.EnableWindow(TRUE);
   }

   return S_OK;
}

STDMETHODIMP COrderPlacingMetaInfoDlg::OnAccountChanged(eAccountChangeType change,
   ICQGAccount* account,
   ICQGPosition* position)
{
   if (account != nullptr && change == eAccountChangeType::actAccountChanged)
   {
      CComBSTR accountName;
      account->get_GWAccountName(&accountName);
      m_lblAccount.SetWindowText(accountName);

      m_spSelectedAccount = account;

      return S_OK;
   }

   return S_FALSE;
}

STDMETHODIMP COrderPlacingMetaInfoDlg::OnInstrumentResolved(BSTR symbol,
   ICQGInstrument* instrument,
   ICQGError* cqg_error)
{
   m_spSubscribedInstrument = instrument;
   CComBSTR fullName;
   m_spSubscribedInstrument->get_FullName(&fullName);
   m_edSymbol.SetWindowText(fullName);

   placeOrder();

   return S_OK;
}

STDMETHODIMP COrderPlacingMetaInfoDlg::OnOrderChanged(eChangeType change,
   ICQGOrder* order,
   ICQGOrderProperties* oldProperties,
   ICQGFill* fill,
   ICQGError* cqgerr)
{
   CComBSTR GWOrderID;
   order->get_GWOrderID(&GWOrderID);
   m_lblGWOrderID.SetWindowText(GWOrderID);

   static const std::unordered_map<eOrderStatus, CComBSTR> orderStatuses = {
      { osActiveAt, "Active At" },
      { osBusted, "Busted" },
      { osCanceled, "Canceled" },
      { osContingent, "Contingent" },
      { osDisconnected, "Disconnected" },
      { osExpired, "Expired" },
      { osFilled, "Filled" },
      { osInCancel, "In Cancel" },
      { osInClient, "In Client" },
      { osInModify, "In Modify" },
      { osInOrderBook, "In Order Book" },
      { osInTransit, "In Transit" },
      { osInTransitTimeout, "In Transit Timeout" },
      { osNotSent, "Not Sent" },
      { osParked, "Parked" },
      { osRejectFCM, "Reject FCM" },
      { osRejectGW, "Reject GW" }
   };

   eOrderStatus status;
   order->get_GWStatus(&status);
   m_lblStatus.SetWindowText(orderStatuses.find(status)->second);

   return S_OK;
}
