// OrderPlacingMetaInfoDlg.cpp : implementation file
//

#include "stdafx.h"
#include <cassert>
#include "OrderPlacingMetaInfo.h"
#include "OrderPlacingMetaInfoDlg.h"
#include "afxdialogex.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


COrderPlacingMetaInfoDlg::COrderPlacingMetaInfoDlg(CWnd* pParent /*=nullptr*/)
   : CDialog(IDD_ORDERPLACINGMETAINFO_DIALOG, pParent)
{
}

void COrderPlacingMetaInfoDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_LBL_ACCOUNT, m_lblAccount);
   DDX_Control(pDX, IDC_LBL_GWORDERID, m_lblGWOrderID);
   DDX_Control(pDX, IDC_ED_SYMBOL, m_edSymbol);
   DDX_Control(pDX, IDC_LBL_STATUS, m_lblStatus);
   DDX_Control(pDX, IDC_CMB_ORDER_TYPE, m_cmbType);
   DDX_Control(pDX, IDC_CMB_ORDER_SIDE, m_cmbSide);
   DDX_Control(pDX, IDC_BTN_PLACE, m_btnPlace);

   DDX_CBString(pDX, IDC_ED_SYMBOL, m_symbol);
   DDX_Text(pDX, IDC_ED_QUANTITY, m_quantity);
   DDX_CBString(pDX, IDC_ED_STOP_PRICE, m_stopPrice);
   DDX_CBString(pDX, IDC_ED_LIMIT_PRICE, m_limitPrice);
}

BEGIN_MESSAGE_MAP(COrderPlacingMetaInfoDlg, CDialog)
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_BN_CLICKED(IDC_BTN_PLACE, &COrderPlacingMetaInfoDlg::OnBnClickedBtnPlace)
END_MESSAGE_MAP()

void COrderPlacingMetaInfoDlg::OnOK()
{
   // Override to prevent dialog from closing on Enter
}

void COrderPlacingMetaInfoDlg::initializeTypesCombo()
{
   int row = m_cmbType.AddString(_T("Market"));
   m_cmbType.SetItemData(row, otMarket);

   row = m_cmbType.AddString(_T("Limit"));
   m_cmbType.SetItemData(row, otLimit);

   row = m_cmbType.AddString(_T("Stop"));
   m_cmbType.SetItemData(row, otStop);

   row = m_cmbType.AddString(_T("StopLimit"));
   m_cmbType.SetItemData(row, otStopLimit);

   // Select first element
   m_cmbType.SetCurSel(0);
}

void COrderPlacingMetaInfoDlg::initializeSidesCombo()
{
   int row = m_cmbSide.AddString(_T("Buy"));
   m_cmbSide.SetItemData(row, otMarket);

   row = m_cmbSide.AddString(_T("Sell"));
   m_cmbSide.SetItemData(row, otLimit);

   // Select first element
   m_cmbSide.SetCurSel(0);
}

BOOL COrderPlacingMetaInfoDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   initializeTypesCombo();
   initializeSidesCombo();
   initializeCQGCEL();

   return TRUE;  // return TRUE  unless you set the focus to a control
}

void COrderPlacingMetaInfoDlg::OnDestroy()
{
   if (m_spCQGCEL)
   {
      HRESULT hr = ICQGCELDispEventImpl::DispEventUnadvise(m_spCQGCEL);
      assert(SUCCEEDED(hr));

      hr = m_spCQGCEL->Shutdown();
   }

   CDialog::OnDestroy();
}

void COrderPlacingMetaInfoDlg::initializeCQGCEL()
{
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), nullptr, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      return;
   }

   configureCQGCEL();

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   m_spCQGCEL->Startup();
}

void COrderPlacingMetaInfoDlg::configureCQGCEL()
{
   ATL::CComPtr<ICQGAPIConfig> spConf;
   m_spCQGCEL->get_APIConfiguration(&spConf);
   spConf->put_ReadyStatusCheck(rscOff);;
   spConf->put_UsedFromATLClient(VARIANT_TRUE);
   spConf->put_UseOrderSide(VARIANT_TRUE);
   spConf->put_NewInstrumentMode(VARIANT_TRUE);
}

void COrderPlacingMetaInfoDlg::switchControls(BOOL value)
{
   m_edSymbol.EnableWindow(value);
   m_btnPlace.EnableWindow(value);
}

void COrderPlacingMetaInfoDlg::subscribe()
{
   UpdateData();

   ATL::CComPtr<ICQGInstrumentSubscriptionRequest> request;
   m_spCQGCEL->CreateInstrumentRequest(&request);
   request->put_Symbol(ATL::CT2W(m_symbol));

   m_spCQGCEL->SubscribeNewInstrument(request);
}

void COrderPlacingMetaInfoDlg::placeOrder()
{
   if (m_spSelectedAccount == nullptr)
   {
      return;
   }

   eOrderType orderType = static_cast<eOrderType>(m_cmbType.GetItemData(m_cmbType.GetCurSel()));
   eOrderSide orderSide = static_cast<eOrderSide>(m_cmbSide.GetItemData(m_cmbSide.GetCurSel()));

   double limitPrice = 0;
   double stopPrice = 0;

   if (orderType == eOrderType::otLimit || orderType == eOrderType::otStopLimit)
   {
      m_spSubscribedInstrument->FromDisplayPrice(ATL::CT2W(m_limitPrice), &limitPrice);
   }

   if (orderType == eOrderType::otStop || orderType == eOrderType::otStopLimit)
   {
      m_spSubscribedInstrument->FromDisplayPrice(ATL::CT2W(m_stopPrice), &stopPrice);
   }

   long quantity = m_quantity;

   ATL::CComPtr<ICQGOrder> spOrder;
   m_spCQGCEL->CreateOrder(orderType, m_spSubscribedInstrument, m_spSelectedAccount,
      quantity, orderSide, limitPrice, stopPrice, /*UEName*/L"", &spOrder);

   spOrder->Place();

   switchControls(TRUE);
}

void COrderPlacingMetaInfoDlg::OnBnClickedBtnPlace()
{
   switchControls(FALSE);
   subscribe();
}
