// OrderPlacingMetaInfoDlg.h : header file
//

#pragma once


class COrderPlacingMetaInfoDlg;

/**
\typedef ATL::IDispEventImpl<1, COrderPlacingMetaInfoDlg, &__uuidof(_ICQGCELEvents), &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
using ICQGCELDispEventImpl = ATL::IDispEventImpl<1, COrderPlacingMetaInfoDlg, &__uuidof(_ICQGCELEvents), &__uuidof(__CQG), 4, 0>;


// COrderPlacingMetaInfoDlg dialog
class COrderPlacingMetaInfoDlg : public CDialog, public ICQGCELDispEventImpl
{
   // Construction
public:
   COrderPlacingMetaInfoDlg(CWnd* pParent = nullptr);	// standard constructor

// Dialog Data
#ifdef AFX_DESIGN_TIME
   enum { IDD = IDD_ORDERPLACINGMETAINFO_DIALOG };
#endif

protected:
   virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support

protected:
   /// \brief Override to prevent dialog from closing on Enter.
   virtual void OnOK();

   /// \brief Used to unadvise from CQGCEL events and shutdown it.
   afx_msg void OnDestroy();

   // Generated message map functions.
   virtual BOOL OnInitDialog();
   afx_msg void OnBnClickedBtnPlace();

   DECLARE_MESSAGE_MAP()

public:
   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(COrderPlacingMetaInfoDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 2, OnGWConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 7, OnAccountChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 17, OnOrderChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 102, OnInstrumentResolved)
   END_SINK_MAP()

protected:
   /// \brief Fired when some changes occurred.
   /// \brief in the connection with the CQG Gateway.
   /// \param newStatus - New connection status
   /// \return S_OK
   STDMETHOD(OnGWConnectionStatusChanged)(
      eConnectionStatus newStatus);

   /// \brief Fired when new account is added, changed or removed.
   /// \param change - Account change type
   /// \param account - Changed account instance
   /// \param position - Changed position instance
   /// \return S_OK
   STDMETHOD(OnAccountChanged)(
      eAccountChangeType change,
      ICQGAccount* account,
      ICQGPosition* position);

   /// \brief Fired when new order was added, order status was 
   /// \brief changed or the order was removed.
   /// \param change - The type of the occurred change
   /// \param order - CQGOrder object representing the order to which the change refers
   /// \param oldProperties - CQGOrderProperties collection representing 
   /// \param the old values of the changed order properties
   /// \param fill - CQGFill object representing the last fill of the order
   /// \param cqgerr - CQGError object to describe the last error, if any
   /// \return S_OK
   STDMETHOD(OnOrderChanged)(
      eChangeType change,
      ICQGOrder* order,
      ICQGOrderProperties* oldProperties,
      ICQGFill* fill,
      ICQGError* cqgerr);

   /// \brief Fired when new instrument is resolved and subscribed or failure happened during a resolving process.
   /// \param symbol - Requested symbol
   /// \param instrument - Subscribed instrument object
   /// \param error - Error occured during a symbol resolution process.
   /// \return S_OK
   STDMETHOD(OnInstrumentResolved)(
      BSTR symbol,
      ICQGInstrument* instrument,
      ICQGError* cqg_error);

private:
   /// \brief Initializes CQGCEL object, and starts the CQGCEL.
   void initializeCQGCEL();

   ///  \brief Configures CQGCEL.
   void configureCQGCEL();

   /// \brief Fills order type combo control with items.
   void initializeTypesCombo();

   /// \brief Fills order side combo control with items.
   void initializeSidesCombo();

   /// \brief Turns controls off/on.
   void switchControls(BOOL value);

   /// \brief Subscribes to instrument.
   void subscribe();

   /// \brief Places order.
   void placeOrder();

private:
   /// \brief CQGCEL object.
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Selected account object.
   ATL::CComPtr<ICQGAccount> m_spSelectedAccount;

   /// \brief Subscribed instrument object.
   ATL::CComPtr<ICQGInstrument> m_spSubscribedInstrument;

   /// \brief Symbol from edit box.
   CString m_symbol;

   /// \brief Stop price presented as string.
   CString m_stopPrice;

   /// \brief Limit price presented as string.
   CString m_limitPrice;

   /// \brief Order quantity.
   int m_quantity;

private:
   /// \brief Account used indicator label control.
   CStatic m_lblAccount;

   /// \brief GWStatus of the last changed order.
   CStatic m_lblStatus;

   /// \brief GWOrderID of the last changed order.
   CStatic m_lblGWOrderID;

   /// \brief Symbol to subscribe. Displays full name after subcription.
   CEdit m_edSymbol;

   /// \brief Order types combo box control.
   CComboBox m_cmbType;

   /// \brief Order side combo box controls.
   CComboBox m_cmbSide;

   /// \brief Place order button control.
   CButton m_btnPlace;
};
